%%================================================================================
%This functionto is to demonstrate image encryption using the reference in
%         [2]. Hua, Zhongyun, et al. "Medical image encryption using high-speed
%              scrambling and pixel adaptive diffusion." Signal Processing 144 (2018): 134-144.
%All copyrights are reserved by Zhongyun Hua. E-mial:huazyum@gmail.com
%All following source code is free to distribute, to use, and to modify
%    for research and study purposes, but absolutely NOT for commercial uses.
%If you use any of the following code in your academic publication(s), 
%    please cite the corresponding paper. 
%If you have any questions, please email me and I will try to response you ASAP.
%It worthwhile to note that all following source code is written under MATLAB R2010a
%    and that files may call built-in functions from specific toolbox(es).
%%=========================================================================

%======================================================================================
%Part A is to simulte the Medical image encryption using bitwise Xor (MIE-BX)
%Part B is to simulte the Medical image encryption using modulo arithmetic (MIE-MA)
%=======================================================================================


%Part A. %%%%%%%%%%%%%%%%%   Demonstration of  MIE-BX    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%=========================================================================
% This demo is to demonstrate the 1. Load plaintext images; 2. encryption; 
% 3. decryption; 4. Analaysis. Inluding 4.1 Histogram; 
% 4.2 the Local Shannon entropy test; 4.3 the NPCR and UACI test; 
% 4.4 Key Sensitivity
%%=========================================================================

%% 1. Load plaintext images
% Image 1
P = imread('cthead-8bit015.tif');
%%=========================================================================

%% 2. Encryption
[C,K] = ImageCipher_BX(P,'en');
%%=========================================================================

%% 3. Decryption
D = ImageCipher_BX(C,'de',K);
%%=========================================================================

%% 4. Analaysis
% 4.1 Histogram
figure,subplot(221),imshow(P,[]),subplot(222),imshow(C,[])
subplot(223),imhist(P),subplot(224),imhist(uint8(C))
%%=========================================================================

% 4.2 the Local Shannon entropy test
%LSE = LocalEntropy(C);
%%=========================================================================

% 4.3 the NPCR and UACI test
% change one bit of a randomly selected pixel 
% to generate a new plaintext image P2
[row, column] = size(P);
r = randi(row,1);
c = randi(column,1);
P2 = P;
if P2(r,c) == 0
    P2(r,c) = P2(r,c)+1;
else
    P2(r,c) = P2(r,c) -1;
end

% encrypte P2 with the same security key K
C2 = ImageCipher_BX(P2,'en',K);

% compute the NPCR value of C and C2;
D = zeros(row,column);
for i = 1:row
    for j = 1:column
        if C2(i,j) ~= C(i,j)
            D(i,j) = 1;
        end
    end
end
NPCR_BX = sum(D(:))/(row*column)*100;

% compute the UACI value of C and C2;
C = double(C);
C2 = double(C2);
A = zeros(row,column);
for i = 1:row
    for j = 1:column
        A(i,j)=abs(C(i,j)-C2(i,j));
    end
end
UACI_BX = sum(A(:))/(255*row*column)*100;
%%=========================================================================

% 4.4 Key Sensitivity
% Load plaintext images
P = imread('cthead-8bit015.tif');

% Generate three similar keys
K = round(rand(1,256));
K2 = K;
K2(200) = 1-K(200);

K3 = K;
K3(240) = 1-K(240);

% Encryption Sensitivity
C = ImageCipher_BX(P,'en',K);
C2 = ImageCipher_BX(P,'en',K2);


% Decryption Sensitivity
D = ImageCipher_BX(C,'de',K);
D2 = ImageCipher_BX(C,'de',K2);
D3 = ImageCipher_BX(C,'de',K3);

figure,
subplot(241),imshow(P,[]),title('P'),subplot(242),imshow(C,[]),title('C_1'),
subplot(243),imshow(C2,[]),title('C_2'),subplot(244),imshow(imabsdiff(C,C2),[]),title('|C_1-C_2|'),
subplot(245),imshow(D,[]),title('D_1'),subplot(246),imshow(D2,[]),title('D_2'),
subplot(247),imshow(D3,[]),title('D_3'),subplot(248),imshow(imabsdiff(D2,D3),[]),title('|D_2-D_3|')




%Part B. %%%%%%%%%%%%%%%%%   Demonstration of  MIE-MA    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%=========================================================================
% This demo is to demonstrate the 1. Load plaintext images; 2. encryption; 
% 3. decryption; 4. Analaysis. Inluding 4.1 Histogram; 
% 4.2 the Local Shannon entropy test; 4.3 the NPCR and UACI test; 
% 4.4 Key Sensitivity
%%=========================================================================

%% 1. Load plaintext images
% Image 1
P = imread('cthead-8bit015.tif');
%%=========================================================================

%% 2. Encryption
[C,K] = ImageCipher_MA(P,'en');
%%=========================================================================

%% 3. Decryption
D = ImageCipher_MA(C,'de',K);
%%=========================================================================

%% 4. Analaysis
% 4.1 Histogram
figure,subplot(221),imshow(P,[]),subplot(222),imshow(C,[])
subplot(223),imhist(P),subplot(224),imhist(uint8(C))
%%=========================================================================

% 4.2 the Local Shannon entropy test
%LSE = LocalEntropy(C);
%%=========================================================================

% 4.3 the NPCR and UACI test
% change one bit of a randomly selected pixel 
% to generate a new plaintext image P2
[row, column] = size(P);
r = randi(row,1);
c = randi(column,1);
P2 = P;
if P2(r,c) == 0
    P2(r,c) = P2(r,c)+1;
else
    P2(r,c) = P2(r,c) -1;
end

% encrypte P2 with the same security key K
C2 = ImageCipher_MA(P2,'en',K);

% compute the NPCR value of C and C2;
D = zeros(row,column);
for i = 1:row
    for j = 1:column
        if C2(i,j) ~= C(i,j)
            D(i,j) = 1;
        end
    end
end
NPCR_MA = sum(D(:))/(row*column)*100;

% compute the UACI value of C and C2;
C = double(C);
C2 = double(C2);
A = zeros(row,column);
for i = 1:row
    for j = 1:column
        A(i,j)=abs(C(i,j)-C2(i,j));
    end
end
UACI_MA = sum(A(:))/(255*row*column)*100;
%%=========================================================================

% 4.4 Key Sensitivity
% Load plaintext images
P = imread('cthead-8bit015.tif');

% Generate three similar keys
K = round(rand(1,256));
K2 = K;
K2(200) = 1-K(200);

K3 = K;
K3(240) = 1-K(240);

% Encryption Sensitivity
C = ImageCipher_MA(P,'en',K);
C2 = ImageCipher_MA(P,'en',K2);


% Decryption Sensitivity
D = ImageCipher_MA(C,'de',K);
D2 = ImageCipher_MA(C,'de',K2);
D3 = ImageCipher_MA(C,'de',K3);

figure,
subplot(241),imshow(P,[]),title('P'),subplot(242),imshow(C,[]),title('C_1'),
subplot(243),imshow(C2,[]),title('C_2'),subplot(244),imshow(imabsdiff(C,C2),[]),title('|C_1-C_2|'),
subplot(245),imshow(D,[]),title('D_1'),subplot(246),imshow(D2,[]),title('D_2'),
subplot(247),imshow(D3,[]),title('D_3'),subplot(248),imshow(imabsdiff(D2,D3),[]),title('|D_2-D_3|')

